@@ -451,7 +451,7 @@ pub(crate) fn apply_line_number_color(
 
 pub fn header(
     display_path: &str,
-    old_path: &Option<String>,
+    extra_info: Option<&String>,
     hunk_num: usize,
     hunk_total: usize,
     file_format: &FileFormat,
@@ -470,16 +470,22 @@ pub fn header(
         hunk_num,
     );
 
-    match old_path {
-        Some(old_path) => {
-            let renamed = format!("Renamed {} to {}", old_path, display_path);
-            format!(
-                "{}\n{} --- {}{}",
-                renamed, display_path, divider, file_format
-            )
+    let mut trailer = format!(" --- {}{}", divider, file_format);
+    if display_options.use_color {
+        trailer = trailer.dimmed().to_string();
+    }
+
+    match extra_info {
+        Some(extra_info) if hunk_num == 1 => {
+            let mut extra_info = extra_info.clone();
+            if display_options.use_color {
+                extra_info = extra_info.dimmed().to_string();
+            }
+
+            format!("{}{}\n{}", display_path_pretty, trailer, extra_info)
         }
-        None => {
-            format!("{} --- {}{}", display_path_pretty, divider, file_format)
+        _ => {
+            format!("{}{}", display_path_pretty, trailer)
         }
     }
 }
