@@ -23,6 +23,7 @@
 // correct.
 #![allow(clippy::mutable_key_type)]
 
+mod conflicts;
 mod constants;
 mod diff;
 mod display;
@@ -35,20 +36,25 @@ mod options;
 mod parse;
 mod positions;
 mod summary;
+mod version;
 
 #[macro_use]
 extern crate log;
 
+use crate::conflicts::START_LHS_MARKER;
 use crate::diff::{dijkstra, unchanged};
 use crate::display::hunks::{matched_pos_to_hunks, merge_adjacent};
+use crate::exit_codes::EXIT_BAD_ARGUMENTS;
 use crate::parse::guess_language::language_globs;
 use crate::parse::syntax;
+use conflicts::apply_conflict_markers;
 use diff::changes::ChangeMap;
 use diff::dijkstra::ExceededGraphLimit;
 use display::context::opposite_positions;
 use exit_codes::{EXIT_FOUND_CHANGES, EXIT_SUCCESS};
 use files::{
-    guess_content, read_files_or_die, read_or_die, relative_paths_in_either, ProbableFileKind,
+    guess_content, read_file_or_die, read_files_or_die, read_or_die, relative_paths_in_either,
+    ProbableFileKind,
 };
 use log::info;
 use mimalloc::MiMalloc;
@@ -62,6 +68,7 @@ use parse::guess_language::{guess, language_name, Language, LanguageOverride};
 static GLOBAL: MiMalloc = MiMalloc;
 
 use diff::sliders::fix_all_sliders;
+use humansize::{format_size, BINARY};
 use options::{DiffOptions, DisplayMode, DisplayOptions, FileArgument, Mode};
 use owo_colors::OwoColorize;
 use rayon::prelude::*;
@@ -147,7 +154,7 @@ fn main() {
             use_color,
             language_overrides,
         } => {
-            for (glob, lang_override) in language_overrides {
+            for (lang_override, globs) in language_overrides {
                 let mut name = match lang_override {
                     LanguageOverride::Language(lang) => language_name(lang),
                     LanguageOverride::PlainText => "Text",
@@ -157,7 +164,10 @@ fn main() {
                     name = name.bold().to_string();
                 }
                 println!("{} (from override)", name);
-                println!(" {}", glob.as_str());
+                for glob in globs {
+                    print!(" {}", glob.as_str());
+                }
+                println!();
             }
 
             for language in Language::iter() {
@@ -173,6 +183,31 @@ fn main() {
                 println!();
             }
         }
+        Mode::DiffFromConflicts {
+            display_path,
+            path,
+            diff_options,
+            display_options,
+            set_exit_code,
+            language_overrides,
+        } => {
+            let diff_result = diff_conflicts_file(
+                &display_path,
+                &path,
+                &display_options,
+                &diff_options,
+                &language_overrides,
+            );
+
+            print_diff_result(&display_options, &diff_result);
+
+            let exit_code = if set_exit_code && diff_result.has_reportable_change() {
+                EXIT_FOUND_CHANGES
+            } else {
+                EXIT_SUCCESS
+            };
+            std::process::exit(exit_code);
+        }
         Mode::Diff {
             diff_options,
             display_options,
@@ -201,38 +236,55 @@ fn main() {
                     options::FileArgument::NamedPath(lhs_path),
                     options::FileArgument::NamedPath(rhs_path),
                 ) if lhs_path.is_dir() && rhs_path.is_dir() => {
-                    // We want to diff files in the directory in
-                    // parallel, but print the results serially (to
-                    // prevent display interleaving).
-                    // https://github.com/rayon-rs/rayon/issues/210#issuecomment-551319338
-                    let (send, recv) = std::sync::mpsc::sync_channel(1);
-
                     let encountered_changes = encountered_changes.clone();
-                    let print_options = display_options.clone();
-
-                    let printing_thread = std::thread::spawn(move || {
-                        for diff_result in recv.into_iter() {
-                            print_diff_result(&print_options, &diff_result);
-
-                            if diff_result.has_reportable_change() {
-                                encountered_changes.store(true, Ordering::Relaxed);
-                            }
-                        }
-                    });
-
-                    diff_directories(
+                    let diff_iter = diff_directories(
                         lhs_path,
                         rhs_path,
                         &display_options,
                         &diff_options,
                         &language_overrides,
-                    )
-                    .try_for_each_with(send, |s, diff_result| s.send(diff_result))
-                    .expect("Receiver should be connected");
+                    );
 
-                    printing_thread
-                        .join()
-                        .expect("Printing thread should not panic");
+                    if matches!(display_options.display_mode, DisplayMode::Json) {
+                        let results = diff_iter
+                            .map(|diff_result| {
+                                if diff_result.has_reportable_change() {
+                                    encountered_changes.store(true, Ordering::Relaxed);
+                                }
+
+                                diff_result
+                            })
+                            .collect();
+
+                        display::json::print_directory(results);
+                    } else {
+                        // We want to diff files in the directory in
+                        // parallel, but print the results serially (to
+                        // prevent display interleaving).
+                        // https://github.com/rayon-rs/rayon/issues/210#issuecomment-551319338
+                        let (send, recv) = std::sync::mpsc::sync_channel(1);
+
+                        let encountered_changes = encountered_changes.clone();
+                        let print_options = display_options.clone();
+
+                        let printing_thread = std::thread::spawn(move || {
+                            for diff_result in recv.into_iter() {
+                                print_diff_result(&print_options, &diff_result);
+
+                                if diff_result.has_reportable_change() {
+                                    encountered_changes.store(true, Ordering::Relaxed);
+                                }
+                            }
+                        });
+
+                        diff_iter
+                            .try_for_each_with(send, |s, diff_result| s.send(diff_result))
+                            .expect("Receiver should be connected");
+
+                        printing_thread
+                            .join()
+                            .expect("Printing thread should not panic");
+                    }
                 }
                 _ => {
                     let diff_result = diff_file(
@@ -245,11 +297,18 @@ fn main() {
                         false,
                         &language_overrides,
                     );
-                    print_diff_result(&display_options, &diff_result);
-
                     if diff_result.has_reportable_change() {
                         encountered_changes.store(true, Ordering::Relaxed);
                     }
+
+                    match display_options.display_mode {
+                        DisplayMode::Inline
+                        | DisplayMode::SideBySide
+                        | DisplayMode::SideBySideShowBoth => {
+                            print_diff_result(&display_options, &diff_result);
+                        }
+                        DisplayMode::Json => display::json::print(&diff_result),
+                    }
                 }
             }
 
@@ -263,84 +322,22 @@ fn main() {
     };
 }
 
-fn format_num_bytes(num_bytes: usize) -> String {
-    if num_bytes >= 1024 * 1024 * 1024 {
-        let g = num_bytes as f64 / (1024.0 * 1024.0 * 1024.0);
-        return format!("{}GiB", g.round());
-    } else if num_bytes >= 1024 * 1024 {
-        let m = num_bytes as f64 / (1024.0 * 1024.0);
-        return format!("{}MiB", m.round());
-    } else if num_bytes >= 1024 {
-        let k = num_bytes as f64 / 1024.0;
-        return format!("{}KiB", k.round());
-    }
-
-    format!("{}B", num_bytes)
-}
-
 /// Print a diff between two files.
 fn diff_file(
     display_path: &str,
-    old_path: Option<String>,
+    extra_info: Option<String>,
     lhs_path: &FileArgument,
     rhs_path: &FileArgument,
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     missing_as_empty: bool,
-    overrides: &[(glob::Pattern, LanguageOverride)],
+    overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
-    diff_file_content(
-        display_path,
-        old_path,
-        lhs_path,
-        rhs_path,
-        &lhs_bytes,
-        &rhs_bytes,
-        display_options,
-        diff_options,
-        overrides,
-    )
-}
-
-fn check_only_text(
-    file_format: &FileFormat,
-    display_path: &str,
-    old_path: Option<String>,
-    lhs_src: &str,
-    rhs_src: &str,
-) -> DiffResult {
-    let has_changes = lhs_src != rhs_src;
-
-    DiffResult {
-        display_path: display_path.to_string(),
-        old_path,
-        file_format: file_format.clone(),
-        lhs_src: FileContent::Text(lhs_src.into()),
-        rhs_src: FileContent::Text(rhs_src.into()),
-        lhs_positions: vec![],
-        rhs_positions: vec![],
-        hunks: vec![],
-        has_byte_changes: has_changes,
-        has_syntactic_changes: has_changes,
-    }
-}
-
-fn diff_file_content(
-    display_path: &str,
-    old_path: Option<String>,
-    _lhs_path: &FileArgument,
-    rhs_path: &FileArgument,
-    lhs_bytes: &[u8],
-    rhs_bytes: &[u8],
-    display_options: &DisplayOptions,
-    diff_options: &DiffOptions,
-    overrides: &[(glob::Pattern, LanguageOverride)],
-) -> DiffResult {
-    let (lhs_src, rhs_src) = match (guess_content(lhs_bytes), guess_content(rhs_bytes)) {
+    let (lhs_src, rhs_src) = match (guess_content(&lhs_bytes), guess_content(&rhs_bytes)) {
         (ProbableFileKind::Binary, _) | (_, ProbableFileKind::Binary) => {
             return DiffResult {
-                old_path,
+                extra_info,
                 display_path: display_path.to_owned(),
                 file_format: FileFormat::Binary,
                 lhs_src: FileContent::Binary,
@@ -355,6 +352,111 @@ fn diff_file_content(
         (ProbableFileKind::Text(lhs_src), ProbableFileKind::Text(rhs_src)) => (lhs_src, rhs_src),
     };
 
+    diff_file_content(
+        display_path,
+        extra_info,
+        lhs_path,
+        rhs_path,
+        &lhs_src,
+        &rhs_src,
+        display_options,
+        diff_options,
+        overrides,
+    )
+}
+
+fn diff_conflicts_file(
+    display_path: &str,
+    path: &FileArgument,
+    display_options: &DisplayOptions,
+    diff_options: &DiffOptions,
+    overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+) -> DiffResult {
+    let bytes = read_file_or_die(path);
+    let src = match guess_content(&bytes) {
+        ProbableFileKind::Text(src) => src,
+        ProbableFileKind::Binary => {
+            eprintln!("error: Expected a text file with conflict markers, got a binary file.");
+            std::process::exit(EXIT_BAD_ARGUMENTS);
+        }
+    };
+
+    let conflict_files = match apply_conflict_markers(&src) {
+        Ok(cf) => cf,
+        Err(msg) => {
+            eprintln!("error: {}", msg);
+            std::process::exit(EXIT_BAD_ARGUMENTS);
+        }
+    };
+
+    if conflict_files.num_conflicts == 0 {
+        eprintln!(
+            "warning: Expected a file with conflict markers {}, but none were found. See --help for usage instructions.\n",
+            START_LHS_MARKER,
+        );
+    }
+
+    let lhs_name = match conflict_files.lhs_name {
+        Some(name) => format!("'{}'", name),
+        None => "the left file".to_owned(),
+    };
+    let rhs_name = match conflict_files.rhs_name {
+        Some(name) => format!("'{}'", name),
+        None => "the right file".to_owned(),
+    };
+
+    let extra_info = format!(
+        "Showing the result of replacing every conflict in {} with {}.",
+        lhs_name, rhs_name
+    );
+
+    diff_file_content(
+        display_path,
+        Some(extra_info),
+        path,
+        path,
+        &conflict_files.lhs_content,
+        &conflict_files.rhs_content,
+        display_options,
+        diff_options,
+        overrides,
+    )
+}
+
+fn check_only_text(
+    file_format: &FileFormat,
+    display_path: &str,
+    extra_info: Option<String>,
+    lhs_src: &str,
+    rhs_src: &str,
+) -> DiffResult {
+    let has_changes = lhs_src != rhs_src;
+
+    DiffResult {
+        display_path: display_path.to_string(),
+        extra_info,
+        file_format: file_format.clone(),
+        lhs_src: FileContent::Text(lhs_src.into()),
+        rhs_src: FileContent::Text(rhs_src.into()),
+        lhs_positions: vec![],
+        rhs_positions: vec![],
+        hunks: vec![],
+        has_byte_changes: has_changes,
+        has_syntactic_changes: has_changes,
+    }
+}
+
+fn diff_file_content(
+    display_path: &str,
+    extra_info: Option<String>,
+    _lhs_path: &FileArgument,
+    rhs_path: &FileArgument,
+    lhs_src: &str,
+    rhs_src: &str,
+    display_options: &DisplayOptions,
+    diff_options: &DiffOptions,
+    overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+) -> DiffResult {
     let (guess_src, guess_path) = match rhs_path {
         FileArgument::NamedPath(path) => (&rhs_src, Path::new(path)),
         FileArgument::Stdin => (&rhs_src, Path::new(&display_path)),
@@ -364,7 +466,7 @@ fn diff_file_content(
     let language = guess(guess_path, guess_src, overrides);
     let lang_config = language.map(tsp::from_language);
 
-    if lhs_bytes == rhs_bytes {
+    if lhs_src == rhs_src {
         let file_format = match language {
             Some(language) => FileFormat::SupportedLanguage(language),
             None => FileFormat::PlainText,
@@ -373,7 +475,7 @@ fn diff_file_content(
         // If the two files are completely identical, return early
         // rather than doing any more work.
         return DiffResult {
-            old_path,
+            extra_info,
             display_path: display_path.to_string(),
             file_format,
             lhs_src: FileContent::Text("".into()),
@@ -390,20 +492,20 @@ fn diff_file_content(
         None => {
             let file_format = FileFormat::PlainText;
             if diff_options.check_only {
-                return check_only_text(&file_format, display_path, old_path, &lhs_src, &rhs_src);
+                return check_only_text(&file_format, display_path, extra_info, lhs_src, rhs_src);
             }
 
-            let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
-            let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+            let lhs_positions = line_parser::change_positions(lhs_src, rhs_src);
+            let rhs_positions = line_parser::change_positions(rhs_src, lhs_src);
             (file_format, lhs_positions, rhs_positions)
         }
         Some(ts_lang) => {
             let arena = Arena::new();
-            match tsp::to_tree_with_limit(diff_options, &ts_lang, &lhs_src, &rhs_src) {
+            match tsp::to_tree_with_limit(diff_options, &ts_lang, lhs_src, rhs_src) {
                 Ok((lhs_tree, rhs_tree)) => {
                     match tsp::to_syntax_with_limit(
-                        &lhs_src,
-                        &rhs_src,
+                        lhs_src,
+                        rhs_src,
                         &lhs_tree,
                         &rhs_tree,
                         &arena,
@@ -419,11 +521,11 @@ fn diff_file_content(
 
                                 let has_syntactic_changes = lhs != rhs;
                                 return DiffResult {
-                                    old_path,
+                                    extra_info,
                                     display_path: display_path.to_string(),
                                     file_format,
-                                    lhs_src: FileContent::Text(lhs_src),
-                                    rhs_src: FileContent::Text(rhs_src),
+                                    lhs_src: FileContent::Text(lhs_src.to_owned()),
+                                    rhs_src: FileContent::Text(rhs_src.to_owned()),
                                     lhs_positions: vec![],
                                     rhs_positions: vec![],
                                     hunks: vec![],
@@ -460,10 +562,8 @@ fn diff_file_content(
                             }
 
                             if exceeded_graph_limit {
-                                let lhs_positions =
-                                    line_parser::change_positions(&lhs_src, &rhs_src);
-                                let rhs_positions =
-                                    line_parser::change_positions(&rhs_src, &lhs_src);
+                                let lhs_positions = line_parser::change_positions(lhs_src, rhs_src);
+                                let rhs_positions = line_parser::change_positions(rhs_src, lhs_src);
                                 (
                                     FileFormat::TextFallback {
                                         reason: "exceeded DFT_GRAPH_LIMIT".into(),
@@ -484,11 +584,11 @@ fn diff_file_content(
 
                                 if diff_options.ignore_comments {
                                     let lhs_comments =
-                                        tsp::comment_positions(&lhs_tree, &lhs_src, &ts_lang);
+                                        tsp::comment_positions(&lhs_tree, lhs_src, &ts_lang);
                                     lhs_positions.extend(lhs_comments);
 
                                     let rhs_comments =
-                                        tsp::comment_positions(&rhs_tree, &rhs_src, &ts_lang);
+                                        tsp::comment_positions(&rhs_tree, rhs_src, &ts_lang);
                                     rhs_positions.extend(rhs_comments);
                                 }
 
@@ -512,35 +612,38 @@ fn diff_file_content(
                                 return check_only_text(
                                     &file_format,
                                     display_path,
-                                    old_path,
-                                    &lhs_src,
-                                    &rhs_src,
+                                    extra_info,
+                                    lhs_src,
+                                    rhs_src,
                                 );
                             }
 
-                            let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
-                            let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+                            let lhs_positions = line_parser::change_positions(lhs_src, rhs_src);
+                            let rhs_positions = line_parser::change_positions(rhs_src, lhs_src);
                             (file_format, lhs_positions, rhs_positions)
                         }
                     }
                 }
                 Err(tsp::ExceededByteLimit(num_bytes)) => {
                     let file_format = FileFormat::TextFallback {
-                        reason: format!("{} exceeded DFT_BYTE_LIMIT", &format_num_bytes(num_bytes)),
+                        reason: format!(
+                            "{} exceeded DFT_BYTE_LIMIT",
+                            &format_size(num_bytes, BINARY)
+                        ),
                     };
 
                     if diff_options.check_only {
                         return check_only_text(
                             &file_format,
                             display_path,
-                            old_path,
-                            &lhs_src,
-                            &rhs_src,
+                            extra_info,
+                            lhs_src,
+                            rhs_src,
                         );
                     }
 
-                    let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
-                    let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+                    let lhs_positions = line_parser::change_positions(lhs_src, rhs_src);
+                    let rhs_positions = line_parser::change_positions(rhs_src, lhs_src);
                     (file_format, lhs_positions, rhs_positions)
                 }
             }
@@ -562,11 +665,11 @@ fn diff_file_content(
     let has_syntactic_changes = !hunks.is_empty();
 
     DiffResult {
-        old_path,
+        extra_info,
         display_path: display_path.to_string(),
         file_format,
-        lhs_src: FileContent::Text(lhs_src),
-        rhs_src: FileContent::Text(rhs_src),
+        lhs_src: FileContent::Text(lhs_src.to_owned()),
+        rhs_src: FileContent::Text(rhs_src.to_owned()),
         lhs_positions,
         rhs_positions,
         hunks,
@@ -586,7 +689,7 @@ fn diff_directories<'a>(
     rhs_dir: &'a Path,
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
-    overrides: &[(glob::Pattern, LanguageOverride)],
+    overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
 ) -> impl ParallelIterator<Item = DiffResult> + 'a {
     let diff_options = diff_options.clone();
     let display_options = display_options.clone();
@@ -627,7 +730,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         "{}",
                         display::style::header(
                             &summary.display_path,
-                            &summary.old_path,
+                            summary.extra_info.as_ref(),
                             1,
                             1,
                             &summary.file_format,
@@ -654,7 +757,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                     "{}",
                     display::style::header(
                         &summary.display_path,
-                        &summary.old_path,
+                        summary.extra_info.as_ref(),
                         1,
                         1,
                         &summary.file_format,
@@ -683,7 +786,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         &summary.rhs_positions,
                         hunks,
                         &summary.display_path,
-                        &summary.old_path,
+                        &summary.extra_info,
                         &summary.file_format,
                     );
                 }
@@ -692,7 +795,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         hunks,
                         display_options,
                         &summary.display_path,
-                        &summary.old_path,
+                        summary.extra_info.as_ref(),
                         &summary.file_format,
                         lhs_src,
                         rhs_src,
@@ -700,6 +803,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         &summary.rhs_positions,
                     );
                 }
+                DisplayMode::Json => unreachable!(),
             }
         }
         (FileContent::Binary, FileContent::Binary) => {
@@ -708,7 +812,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                     "{}",
                     display::style::header(
                         &summary.display_path,
-                        &summary.old_path,
+                        summary.extra_info.as_ref(),
                         1,
                         1,
                         &FileFormat::Binary,
@@ -729,7 +833,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 "{}",
                 display::style::header(
                     &summary.display_path,
-                    &summary.old_path,
+                    summary.extra_info.as_ref(),
                     1,
                     1,
                     &FileFormat::Binary,
@@ -755,8 +859,8 @@ mod tests {
             None,
             &FileArgument::from_path_argument(OsStr::new("foo.el")),
             &FileArgument::from_path_argument(OsStr::new("foo.el")),
-            s.as_bytes(),
-            s.as_bytes(),
+            s,
+            s,
             &DisplayOptions::default(),
             &DiffOptions::default(),
             &[],
@@ -765,19 +869,4 @@ mod tests {
         assert_eq!(res.lhs_positions, vec![]);
         assert_eq!(res.rhs_positions, vec![]);
     }
-
-    #[test]
-    fn test_num_bytes_small() {
-        assert_eq!(&format_num_bytes(200), "200B");
-    }
-
-    #[test]
-    fn test_num_bytes_kb() {
-        assert_eq!(&format_num_bytes(10_000), "10KiB");
-    }
-
-    #[test]
-    fn test_num_bytes_mb() {
-        assert_eq!(&format_num_bytes(3_000_000), "3MiB");
-    }
 }
