@@ -73,7 +73,7 @@ pub enum Language {
     Zig,
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq)]
 pub enum LanguageOverride {
     Language(Language),
     PlainText,
@@ -365,16 +365,18 @@ fn looks_like_hacklang(path: &Path, src: &str) -> bool {
 pub fn guess(
     path: &Path,
     src: &str,
-    overrides: &[(glob::Pattern, LanguageOverride)],
+    overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
 ) -> Option<Language> {
     if let Some(file_name) = path.file_name() {
         let file_name = file_name.to_string_lossy();
-        for (pattern, lang_override) in overrides {
-            if pattern.matches(&file_name) {
-                match lang_override {
-                    LanguageOverride::Language(lang) => return Some(*lang),
-                    LanguageOverride::PlainText => {
-                        return None;
+        for (lang_override, patterns) in overrides {
+            for pattern in patterns {
+                if pattern.matches(&file_name) {
+                    match lang_override {
+                        LanguageOverride::Language(lang) => return Some(*lang),
+                        LanguageOverride::PlainText => {
+                            return None;
+                        }
                     }
                 }
             }
@@ -415,7 +417,7 @@ fn from_emacs_mode_header(src: &str) -> Option<Language> {
             (Some(cap), _) | (_, Some(cap)) => cap[1].into(),
             _ => "".into(),
         };
-        let lang = match mode_name.to_ascii_lowercase().trim().borrow() {
+        let lang = match mode_name.to_ascii_lowercase().trim() {
             "ada" => Some(Ada),
             "c" => Some(C),
             "clojure" => Some(Clojure),
@@ -596,8 +598,8 @@ mod tests {
                 path,
                 "",
                 &[(
-                    glob::Pattern::new("*.el").unwrap(),
-                    LanguageOverride::Language(Css)
+                    LanguageOverride::Language(Css),
+                    vec![glob::Pattern::new("*.el").unwrap()],
                 )]
             ),
             Some(Css)
