@@ -1,4 +1,5 @@
-const DIGITS = token(sep1(/[0-9]+/, /_+/))
+const DIGITS = token(choice('0', seq(/[1-9]/, optional(seq(optional('_'), sep1(/[0-9]+/, /_+/))))))
+const DECIMAL_DIGITS = token(sep1(/[0-9]+/, '_'))
 const HEX_DIGITS = token(sep1(/[A-Fa-f0-9]+/, '_'))
 const PREC = {
   // https://introcs.cs.princeton.edu/java/11precedence/
@@ -106,7 +107,7 @@ module.exports = grammar({
     )),
 
     octal_integer_literal: $ => token(seq(
-      choice('0o', '0O'),
+      choice('0o', '0O', '0'),
       sep1(/[0-7]+/, '_'),
       optional(choice('l', 'L'))
     )),
@@ -118,10 +119,10 @@ module.exports = grammar({
     )),
 
     decimal_floating_point_literal: $ => token(choice(
-      seq(DIGITS, '.', optional(DIGITS), optional(seq((/[eE]/), optional(choice('-', '+')), DIGITS)), optional(/[fFdD]/)),
-      seq('.', DIGITS, optional(seq((/[eE]/), optional(choice('-', '+')), DIGITS)), optional(/[fFdD]/)),
-      seq(DIGITS, /[eEpP]/, optional(choice('-', '+')), DIGITS, optional(/[fFdD]/)),
-      seq(DIGITS, optional(seq((/[eE]/), optional(choice('-', '+')), DIGITS)), (/[fFdD]/))
+      seq(DECIMAL_DIGITS, '.', optional(DECIMAL_DIGITS), optional(seq((/[eE]/), optional(choice('-', '+')), DECIMAL_DIGITS)), optional(/[fFdD]/)),
+      seq('.', DECIMAL_DIGITS, optional(seq((/[eE]/), optional(choice('-', '+')), DECIMAL_DIGITS)), optional(/[fFdD]/)),
+      seq(DIGITS, /[eEpP]/, optional(choice('-', '+')), DECIMAL_DIGITS, optional(/[fFdD]/)),
+      seq(DIGITS, optional(seq((/[eE]/), optional(choice('-', '+')), DECIMAL_DIGITS)), (/[fFdD]/))
     )),
 
     hex_floating_point_literal: $ => token(seq(
@@ -213,11 +214,19 @@ module.exports = grammar({
       $.switch_expression,
     ),
 
-    cast_expression: $ => prec(PREC.CAST, seq(
-      '(',
-      sep1(field('type', $._type), '&'),
-      ')',
-      field('value', $.expression)
+    cast_expression: $ => prec(PREC.CAST, choice(
+      seq(
+        '(',
+        field('type', $._type),
+        ')',
+        field('value', $.expression),
+      ),
+      seq(
+        '(',
+        sep1(field('type', $._type), '&'),
+        ')',
+        field('value', choice($.primary_expression, $.lambda_expression)),
+      ),
     )),
 
     assignment_expression: $ => prec.right(PREC.ASSIGN, seq(
@@ -345,6 +354,8 @@ module.exports = grammar({
 
     parenthesized_expression: $ => seq('(', $.expression, ')'),
 
+    condition: $ => seq('(', $.expression, ')'),
+
     class_literal: $ => prec.dynamic(PREC.CLASS_LITERAL, seq($._unannotated_type, '.', 'class')),
 
     object_creation_expression: $ => choice(
@@ -584,14 +595,14 @@ module.exports = grammar({
 
     if_statement: $ => prec.right(seq(
       'if',
-      field('condition', $.parenthesized_expression),
+      field('condition', $.condition),
       field('consequence', $.statement),
       optional(seq('else', field('alternative', $.statement)))
     )),
 
     while_statement: $ => seq(
       'while',
-      field('condition', $.parenthesized_expression),
+      field('condition', $.condition),
       field('body', $.statement)
     ),
 
